// ---------------------------------------------------------------- //
// Sensor Project
// Adapted by James Kelly
// Written on 3 October 2021
// ---------------------------------------------------------------- //

// Derived from:
// ---------------------------------------------------------------- //
// Arduino Ultrasoninc Sensor HC-SR04
// Re-writed by Arbi Abdul Jabbaar
// Using Arduino IDE 1.8.7
// Using HC-SR04 Module
// Tested on 17 September 2019
// ---------------------------------------------------------------- //

#include <Servo.h>

Servo servo1; // create servo object to control the first servo
Servo servo2; // create servo object to control the second servo

//change values to update initial and end servo positions
int startPos = 90;                     //set initial position between 0 and 180
int endPos = 180;                     //set end position between 0 and 180

//function to perform wave with follower leg
void wave() {
  //increment from startPos to endPos, if startPos > endPos, you will need to change "pos <= endPos" and "pos += 1" -- how should it be changed?
  for (int pos = startPos; pos <= endPos; pos += 1) {
    int pos1 = pos;                        
    int pos2 = 180 - (pos - 90);                   
    
    servo1.write(pos1);               // tell servo1 to go to position in variable 'pos1'
    servo2.write(pos2);               // tell servo2 to go to position in variable 'pos2'
    delay(15);                        // waits 15ms for the servo to reach the position
  }

  //increment from endPos to startPos, if startPos > endPos, you will need to change "pos >= startPos" and "pos -= 1" -- how should it be changed?
  for (int pos = endPos; pos >= startPos; pos -= 1) {
    int pos1 = pos;                       
    int pos2 = 180 - (pos - 90); 
    
    servo1.write(pos1);               // tell servo1 to go to position in variable 'pos1'
    servo2.write(pos2);               // tell servo2 to go to position in variable 'pos2'
    delay(15);                        // waits 15ms for the servo to reach the position
  }

}

#define echoPin 2 // attach pin D2 Arduino to pin Echo of HC-SR04
#define trigPin 3 //attach pin D3 Arduino to pin Trig of HC-SR04

// defines variables
long duration; // variable for the duration of sound wave travel
int priorDistance; // variable for the prior distance measurement
int currentDistance; // variable for the current distance measurement
int thresholdDist;  // variable to be set in the setup to define the threshold for apporaching the sensor

void setup() {
  servo1.attach(9);                     // attaches the first servo on pin 9 to the servo object
  servo2.attach(10);                    // attaches the second servo on pin 10 to the servo object
  
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an OUTPUT
  pinMode(echoPin, INPUT); // Sets the echoPin as an INPUT

  // Calculate the threshold distance for detction of approaching people
  // Clears the trigPin condition
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin HIGH (ACTIVE) for 10 microseconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH);
  // Calculating the distance
  currentDistance = duration * 0.034 / 2; // Speed of sound wave divided by 2 (go and back)

  // set the threshold distance at half the current reading
  thresholdDist = currentDistance / 2;
  // adjust priorDistance var to most recent reading
  priorDistance = currentDistance;

  Serial.begin(9600); // // Serial Communication is starting with 9600 of baudrate speed
  Serial.println("Ultrasonic Sensor HC-SR04 Test"); // print some text in Serial Monitor
  Serial.println("with Arduino UNO R3");
}

void loop() {
  // Clears the trigPin condition
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin HIGH (ACTIVE) for 10 microseconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH);
  // Calculating the distance
  currentDistance = duration * 0.034 / 2; // Speed of sound wave divided by 2 (go and back)

  //checks if someone just walked up
  if(currentDistance < thresholdDist && (currentDistance < priorDistance && priorDistance > thresholdDist)){
    wave(); //if someone did just walk up, wave
  }

  //updates priorDistance var
  priorDistance = currentDistance;

  // Displays the distance on the Serial Monitor (for debugging)
  Serial.print("Distance: ");
  Serial.print(currentDistance);
  Serial.println(" cm");
}
